@@ -12,12 +12,12 @@ from typing import (
     Awaitable,
     Callable,
     Dict,
-    Generator,
+    Iterator,
     List,
     Mapping,
     Optional,
+    Sequence,
     Tuple,
-    Type,
     TYPE_CHECKING,
     Union,
 )
@@ -30,13 +30,15 @@ from hypercorn.typing import (
     WebsocketScope,
 )
 
+from .datastructures import FileStorage
+
 try:
     from typing import Protocol
 except ImportError:
     from typing_extensions import Protocol  # type: ignore
 
 if TYPE_CHECKING:
-    from werkzeug.datastructures import Headers  # noqa: F401
+    from werkzeug.datastructures import Authorization, Headers  # noqa: F401
     from werkzeug.wrappers import Response as WerkzeugResponse
 
     from .app import Quart
@@ -49,11 +51,12 @@ FilePath = Union[bytes, str, os.PathLike]
 ResponseValue = Union[
     "Response",
     "WerkzeugResponse",
-    AnyStr,
+    bytes,
+    str,
     Mapping[str, Any],  # any jsonify-able dict
     List[Any],  # any jsonify-able list
-    AsyncGenerator[AnyStr, None],
-    Generator[AnyStr, None, None],
+    Iterator[bytes],
+    Iterator[str],
 ]
 StatusCode = int
 
@@ -62,7 +65,9 @@ HeaderName = str
 HeaderValue = Union[str, List[str], Tuple[str, ...]]
 
 # the possible types for HTTP headers
-HeadersValue = Union["Headers", Dict[HeaderName, HeaderValue], List[Tuple[HeaderName, HeaderValue]]]
+HeadersValue = Union[
+    "Headers", Mapping[HeaderName, HeaderValue], Sequence[Tuple[HeaderName, HeaderValue]]
+]
 
 # The possible types returned by a route function.
 ResponseReturnValue = Union[
@@ -72,16 +77,17 @@ ResponseReturnValue = Union[
     Tuple[ResponseValue, StatusCode, HeadersValue],
 ]
 
+ResponseTypes = Union["Response", "WerkzeugResponse"]
+
 AppOrBlueprintKey = Optional[str]  # The App key is None, whereas blueprints are named
 AfterRequestCallable = Union[
-    Callable[["Response"], "Response"], Callable[["Response"], Awaitable["Response"]]
+    Callable[[ResponseTypes], ResponseTypes], Callable[[ResponseTypes], Awaitable[ResponseTypes]]
 ]
 AfterServingCallable = Union[Callable[[], None], Callable[[], Awaitable[None]]]
 AfterWebsocketCallable = Union[
-    Callable[["Response"], Optional["Response"]],
-    Callable[["Response"], Awaitable[Optional["Response"]]],
+    Callable[[ResponseTypes], Optional[ResponseTypes]],
+    Callable[[ResponseTypes], Awaitable[Optional[ResponseTypes]]],
 ]
-BeforeFirstRequestCallable = Union[Callable[[], None], Callable[[], Awaitable[None]]]
 BeforeRequestCallable = Union[
     Callable[[], Optional[ResponseReturnValue]],
     Callable[[], Awaitable[Optional[ResponseReturnValue]]],
@@ -92,8 +98,8 @@ BeforeWebsocketCallable = Union[
     Callable[[], Awaitable[Optional[ResponseReturnValue]]],
 ]
 ErrorHandlerCallable = Union[
-    Callable[[Exception], ResponseReturnValue],
-    Callable[[Exception], Awaitable[ResponseReturnValue]],
+    Callable[[Any], ResponseReturnValue],
+    Callable[[Any], Awaitable[ResponseReturnValue]],
 ]
 ShellContextProcessorCallable = Callable[[], Dict[str, Any]]
 TeardownCallable = Union[
@@ -145,7 +151,7 @@ class ASGIWebsocketProtocol(Protocol):
 
 
 class TestHTTPConnectionProtocol(Protocol):
-    push_promises: List[Tuple[str, Headers]]
+    push_promises: list[tuple[str, Headers]]
 
     def __init__(self, app: Quart, scope: HTTPScope, _preserve_context: bool = False) -> None:
         ...
@@ -203,10 +209,10 @@ class TestWebsocketConnectionProtocol(Protocol):
 
 class TestClientProtocol(Protocol):
     app: Quart
-    cookie_jar: Optional[CookieJar]
-    http_connection_class: Type[TestHTTPConnectionProtocol]
-    push_promises: List[Tuple[str, Headers]]
-    websocket_connection_class: Type[TestWebsocketConnectionProtocol]
+    cookie_jar: CookieJar | None
+    http_connection_class: type[TestHTTPConnectionProtocol]
+    push_promises: list[tuple[str, Headers]]
+    websocket_connection_class: type[TestWebsocketConnectionProtocol]
 
     def __init__(self, app: Quart, use_cookies: bool = True) -> None:
         ...
@@ -216,15 +222,19 @@ class TestClientProtocol(Protocol):
         path: str,
         *,
         method: str = "GET",
-        headers: Optional[Union[dict, Headers]] = None,
-        data: Optional[AnyStr] = None,
-        form: Optional[dict] = None,
-        query_string: Optional[dict] = None,
-        json: Any = None,
+        headers: dict | Headers | None = None,
+        data: AnyStr | None = None,
+        form: dict | None = None,
+        files: dict[str, FileStorage] | None = None,
+        query_string: dict | None = None,
+        json: Any,
         scheme: str = "http",
         follow_redirects: bool = False,
         root_path: str = "",
         http_version: str = "1.1",
+        scope_base: dict | None = None,
+        auth: Authorization | tuple[str, str] | None = None,
+        subdomain: str | None = None,
     ) -> Response:
         ...
 
@@ -233,11 +243,14 @@ class TestClientProtocol(Protocol):
         path: str,
         *,
         method: str = "GET",
-        headers: Optional[Union[dict, Headers]] = None,
-        query_string: Optional[dict] = None,
+        headers: dict | Headers | None = None,
+        query_string: dict | None = None,
         scheme: str = "http",
         root_path: str = "",
         http_version: str = "1.1",
+        scope_base: dict | None = None,
+        auth: Authorization | tuple[str, str] | None = None,
+        subdomain: str | None = None,
     ) -> TestHTTPConnectionProtocol:
         ...
 
@@ -245,12 +258,15 @@ class TestClientProtocol(Protocol):
         self,
         path: str,
         *,
-        headers: Optional[Union[dict, Headers]] = None,
-        query_string: Optional[dict] = None,
+        headers: dict | Headers | None = None,
+        query_string: dict | None = None,
         scheme: str = "ws",
-        subprotocols: Optional[List[str]] = None,
+        subprotocols: list[str] | None = None,
         root_path: str = "",
         http_version: str = "1.1",
+        scope_base: dict | None = None,
+        auth: Authorization | tuple[str, str] | None = None,
+        subdomain: str | None = None,
     ) -> TestWebsocketConnectionProtocol:
         ...
 
@@ -283,10 +299,10 @@ class TestClientProtocol(Protocol):
         server_name: str,
         key: str,
         value: str = "",
-        max_age: Optional[Union[int, timedelta]] = None,
-        expires: Optional[Union[int, float, datetime]] = None,
+        max_age: int | timedelta | None = None,
+        expires: int | float | datetime | None = None,
         path: str = "/",
-        domain: Optional[str] = None,
+        domain: str | None = None,
         secure: bool = False,
         httponly: bool = False,
         samesite: str = None,
@@ -295,7 +311,7 @@ class TestClientProtocol(Protocol):
         ...
 
     def delete_cookie(
-        self, server_name: str, key: str, path: str = "/", domain: Optional[str] = None
+        self, server_name: str, key: str, path: str = "/", domain: str | None = None
     ) -> None:
         ...
 
@@ -304,11 +320,11 @@ class TestClientProtocol(Protocol):
         path: str = "/",
         *,
         method: str = "GET",
-        headers: Optional[Union[dict, Headers]] = None,
-        query_string: Optional[dict] = None,
+        headers: dict | Headers | None = None,
+        query_string: dict | None = None,
         scheme: str = "http",
-        data: Optional[AnyStr] = None,
-        form: Optional[dict] = None,
+        data: AnyStr | None = None,
+        form: dict | None = None,
         json: Any = None,
         root_path: str = "",
         http_version: str = "1.1",
@@ -340,3 +356,8 @@ class TestAppProtocol(Protocol):
 
     async def __aexit__(self, exc_type: type, exc_value: BaseException, tb: TracebackType) -> None:
         ...
+
+
+class Event(Protocol):
+    def is_set(self) -> bool:
+        ...
